@@ -36,10 +36,12 @@ namespace trng {
 
     // Uniform random number generator concept
     typedef unsigned long long result_type;
-    result_type operator()() const;  
+    result_type operator()() const;
     static const result_type min=0;
     static const result_type max=18446744073709551615ull;
-
+  private:
+    static const bool use_mask=math::numeric_limits<result_type>::digits>64;
+  public:
     // Parameter and status classes
     class parameter_type;
     class status_type;
@@ -222,32 +224,25 @@ namespace trng {
   // Inline and template methods
   
   inline void lcg64_shift::step() const {
-    S.r=P.a*S.r+P.b;
-#if ULONG_LONG_MAX>18446744073709551615ull
-    S.r&=0xfffffffffffffffful;
-#endif
+    if (use_mask)
+      S.r=(P.a*S.r+P.b) & max;
+    else
+      S.r=(P.a*S.r+P.b);
     }
     
   inline lcg64_shift::result_type lcg64_shift::operator()() const {
     step();
     unsigned long long t=S.r;
     t^=(t>>17);
-#if ULONG_LONG_MAX>18446744073709551615ull
-    t&=0xfffffffffffffffful;
-#endif
     t^=(t<<31);
-#if ULONG_LONG_MAX>18446744073709551615ull
-    t&=0xfffffffffffffffful;
-#endif
+    if (use_mask)
+      t&=max;
     t^=(t>>8);
-#if ULONG_LONG_MAX>18446744073709551615ull
-    t&=0xfffffffffffffffful;
-#endif
     return t;
   }
 
   inline long lcg64_shift::operator()(long x) const {
-    return static_cast<long>(utility::uniformco(*this)*x);
+    return static_cast<long>(utility::uniformco<double, lcg64_shift>(*this)*x);
   }
 
 }
