@@ -1,4 +1,4 @@
-// Copyright (C) 2000-2007 Heiko Bauke <heiko.bauke@mpi-hd.mpg.de>
+// Copyright (C) 2000-2008 Heiko Bauke <heiko.bauke@mpi-hd.mpg.de>
 //  
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License in
@@ -19,12 +19,10 @@
 
 namespace trng {
 
-  // Uniform random number generator concept
-  const minstd::result_type minstd::min=1l;
-  const minstd::result_type minstd::max=2147483646l;
-  
   // Parameter and status classes
   
+  // Uniform random number generator concept
+
   // Equality comparable concept
   bool operator==(const minstd::status_type &S1, 
 		  const minstd::status_type &S2) {
