@@ -37,7 +37,7 @@ namespace trng {
     static const result_type modulus=2147483647l;
   public:
     static const result_type min=0l;
-    static const result_type max=2147483646l;
+    static const result_type max=modulus-1;
 
     // Parameter and status classes
     class parameter_type;
@@ -224,12 +224,7 @@ namespace trng {
 			 static_cast<unsigned long long>(S.r1)+
 			 static_cast<unsigned long long>(P.a2)*
 			 static_cast<unsigned long long>(S.r2));
-    t=(t&0x7fffffffull)+(t>>31);
-    if (t>=2ull*2147483647ull)
-      t-=2ull*2147483647ull;
-    if (t>=2147483647ull)
-      t-=2147483647ull;
-    S.r2=S.r1;  S.r1=t;
+    S.r2=S.r1;  S.r1=utility::modulo<modulus, 2>(t);
   }
   
   inline mrg2::result_type mrg2::operator()() const {
@@ -238,7 +233,7 @@ namespace trng {
   }
   
   inline long mrg2::operator()(long x) const {
-    return static_cast<long>(utility::uniformco(*this)*x);
+    return static_cast<long>(utility::uniformco<double, mrg2>(*this)*x);
   }
 
 }
