@@ -36,9 +36,10 @@ namespace trng {
     result_type operator()() const;
   private:
     static const result_type modulus=2147462579l;  // 2^31-21069
+    static const result_type gen=1616076847l;
   public:
     static const result_type min=0l;
-    static const result_type max=2147462578l;
+    static const result_type max=modulus-1;
 
     // Parameter and status classes
     class parameter_type;
@@ -46,14 +47,13 @@ namespace trng {
 
     class parameter_type {
       result_type a1, a2, a3;
-      utility::power<2147462579l> g;
+      static utility::power<yarn3s::modulus, yarn3s::gen> g;
     public:
       parameter_type() :
-        a1(0), a2(0), a3(0), g() { };
+        a1(0), a2(0), a3(0) { };
       parameter_type(result_type a1, result_type a2,
-                     result_type a3,
-		     result_type g) :
-	a1(a1), a2(a2), a3(a3), g(g) { };
+                     result_type a3) :
+	a1(a1), a2(a2), a3(a3) { };
 
       friend class yarn3s;
 
@@ -70,7 +70,7 @@ namespace trng {
         out.flags(std::ios_base::dec | std::ios_base::fixed |
                   std::ios_base::left);
         out << '('
-            << P.a1 << ' ' << P.a2 << ' ' << P.a3 << ' ' << P.g
+            << P.a1 << ' ' << P.a2 << ' ' << P.a3
             << ')';
         out.flags(flags);
         return out;
@@ -87,8 +87,7 @@ namespace trng {
         in >> utility::delim('(')
            >> P_new.a1 >> utility::delim(' ')
            >> P_new.a2 >> utility::delim(' ')
-           >> P_new.a3 >> utility::delim(' ')
-           >> P_new.g  >> utility::delim(')');
+           >> P_new.a3 >> utility::delim(')');
         if (in)
           P=P_new;
         in.flags(flags);
@@ -237,11 +236,7 @@ namespace trng {
 			 static_cast<unsigned long long>(S.r2)+
 			 static_cast<unsigned long long>(P.a3)*
 			 static_cast<unsigned long long>(S.r3));
-    t=(t&0x7fffffffull)+(t>>31)*21069;
-    t=(t&0x7fffffffull)+(t>>31)*21069;
-    if (t>=2147462579ull)
-      t-=2147462579ull;
-    S.r3=S.r2;  S.r2=S.r1;  S.r1=t;
+    S.r3=S.r2;  S.r2=S.r1;  S.r1=utility::modulo<modulus, 3>(t);
   }
   
   inline yarn3s::result_type yarn3s::operator()() const {
@@ -250,7 +245,7 @@ namespace trng {
   }
   
   inline long yarn3s::operator()(long x) const {
-    return static_cast<long>(utility::uniformco(*this)*x);
+    return static_cast<long>(utility::uniformco<double, yarn3s>(*this)*x);
   }
 
 }
